<?php
	
	function sti_mssql_parse_parameters($query)
	{
		$result = "";
		while (strpos($query, "{") !== false)
		{
			$result .= substr($query, 0, strpos($query, "{"));
			$query = substr($query, strpos($query, "{") + 1);
			$parameter_name = substr($query, 0, strpos($query, "}"));
			$result .= sti_get_parameter($parameter_name);
			$query = substr($query, strpos($query, "}") + 1);
		}
		
		return $result.$query;
	}
	
	function sti_mssql_parse_connection_string($connection_string)
	{
		class connection_info {
			var $host;
			var $database;
			var $user_id;
			var $password;
		}

		$info = new connection_info;
		$parameters = split(";", $connection_string);
		foreach($parameters as $parameter)
		{
			$parts = split("=", $parameter);
			$name = $parts[0];
			$value = $parts[1];
			
			if (isset($value))
			{
				switch (strtolower($name))
				{
					case "data source":
					case "server":
						$info->host = $value;
						break;

					case "database":
					case "initial catalog":
						$info->database = $value;
						break;

					case "user":
					case "userid":
					case "user id":
						$info->user_id = $value;
						break;

					case "password":
					case "pwd":
						$info->password = $value;
						break;
				}
			}
		}

		return $info;
	}

	function sti_mssql_get_column_type($type)
	{
		switch (strtolower($type))
		{
			case "numeric":
			case "int":
				return "System.Int32";
				
			case "datetime":
				return "System.DateTime";

			case "real":
			case "money":
				return "System.Decimal";
		}

		return "System.String";
	}
	
	function sti_mssql_test_connection($connection_string)
	{
		$info = sti_mssql_parse_connection_string($connection_string);
		$link = mssql_connect($info->host, $info->user_id, $info->password) or die("ServerError:Could not connect to host '$info->host'.");
		mssql_close($link);
		
		return "Successfull";
	}

	function sti_mssql_get_columns($connection_string, $query)
	{
		$info = sti_mssql_parse_connection_string($connection_string);
		$link = mssql_connect($info->host, $info->user_id, $info->password) or die("ServerError:Could not connect to host '$info->host'.");
		mssql_select_db($info->database, $link) or die("ServerError:Could not find database '$info->database'.");
		$result = mssql_query($query, $link) or die("ServerError:Data not found.");

		$xml_output = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<Tables>\n  <RetrieveColumns>\n";

		while ($column = mssql_fetch_field($result))
		{
			$column_type = sti_mssql_get_column_type($column->type);
			$xml_output .= "    <Column table=\"$column->table\" name=\"$column->name\" type=\"$column_type\" />\n";
		}
		$xml_output .= "  </RetrieveColumns>\n</Tables>";

		mssql_free_result($result);
		mssql_close($link);

		return $xml_output;
	}

	function sti_mssql_get_data($connection_string, $data_source_name, $query)
	{
		$info = sti_mssql_parse_connection_string($connection_string);
		$link = mssql_connect($info->host, $info->user_id, $info->password) or die("ServerError:Could not connect to host '$info->host'.");
		mssql_select_db($info->database, $link) or die("ServerError:Could not find database '$info->database'.");
		
		$query = sti_mssql_parse_parameters($query);
		$result = mssql_query($query, $link) or die("ServerError:Data not found.");

		$xml_output = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<Database>\n";

		$columns = Array();
		while ($column = mssql_fetch_field($result))
		{
			array_push($columns, $column);
		}

		while ($row = mssql_fetch_assoc($result))
		{
			$xml_output .= "  <$data_source_name>\n";
			foreach($columns as $column)
			{
				$value = $row[$column->name];
				$value = str_replace("&", "&amp;", $value);
				$value = str_replace("<", "&lt;", $value);
				$value = str_replace(">", "&gt;", $value);

				$xml_output .= "    <$column->name>$value</$column->name>\n";
			}
			$xml_output .= "  </$data_source_name>\n";
		}
		
		$xml_output .= "</Database>";

		mssql_free_result($result);
		mssql_close($link);

		return $xml_output;
	}

?>