<?php

	function sti_pg_parse_parameters($query)
	{
		$result = "";
		while (strpos($query, "{") !== false)
		{
			$result .= substr($query, 0, strpos($query, "{"));
			$query = substr($query, strpos($query, "{") + 1);
			$parameter_name = substr($query, 0, strpos($query, "}"));
			$result .= sti_get_parameter($parameter_name);
			$query = substr($query, strpos($query, "}") + 1);
		}
		
		return $result.$query;
	}
	
	function sti_pg_parse_connection_string($connection_string)
    {
        class connection_info
		{
            var $host;
            var $port = "5432";
            var $database;
            var $user_id;
            var $password;
        }
		
		$info = new connection_info;
		$parameters = split(";", $connection_string);
		foreach($parameters as $parameter)
		{
			$parts = split("=", $parameter);
			$name = $parts[0];
			$value = $parts[1];

			if (isset($value))
			{
				switch (strtolower($name))
				{
					case "server":
					case "host":
					case "location":
						$info->host = $value;
						break;

					case "port":
						$info->port = $value;
						break;

					case "database":
					case "datasource":
					case "data source":
						$info->database = $value;
						break;

					case "uid":
					case "user":
					case "userid":
					case "user id":
						$info->user_id = $value;
						break;

					case "pwd":
					case "password":
						$info->password = $value;
						break;
				}
			}
		}

		return $info;
	}

	function sti_pg_get_column_type($type)
    {
        switch (strtolower($type))
        {
            case "varchar":
            case "bpchar":
            case "macaddr":
            case "inet":
            case "money":
            case "text":
            case "interval":
            case "bit":
            case "varbit":
            case "bytea":
                return "System.String";

            case "bool":
                return "System.Boolean";

            case "int8":
            case "int4":
            case "int2":
                return "System.Int32";

            case "date":
            case "time":
            case "timetz":
            case "timestamp":
            case "timestamptz":
                return "System.DateTime";
        }

        return "System.Decimal";
    }

	function sti_pg_test_connection($connection_string)
	{
		$info = sti_pg_parse_connection_string($connection_string);
		$link = pg_connect("host=$info->host port=$info->port dbname=$info->database user=$info->user_id password=$info->password") or die("ServerError:Could not connect to host '$info->host', database '$info->database'.");
		pg_close($link);
		
		return "Successfull";
	}
	
	function sti_pg_get_columns($connection_string, $query)
	{
		$info = sti_pg_parse_connection_string($connection_string);
		$link = pg_connect("host=$info->host port=$info->port dbname=$info->database user=$info->user_id password=$info->password") or die("ServerError:Could not connect to host '$info->host', database '$info->database'.");
		$result = pg_query($link, $query) or die("ServerError:Data not found.");

		$xml_output = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<Tables>\n  <RetrieveColumns>\n";

		$count = pg_num_fields($result);
		for ($fid = 0; $fid < $count; $fid++)
		{
			$field_table = pg_field_table($result, $fid);
			$field_name = pg_field_name($result, $fid);
			$field_type = pg_field_type($result, $fid);
			$column_type = sti_pg_get_column_type($field_type);
			$xml_output .= "    <Column table='$field_table' name='$field_name' type='$column_type' />\n";
		}
		
		$xml_output .= "  </RetrieveColumns>\n</Tables>";
		
		pg_free_result($result);
		pg_close($link);
		
		return $xml_output;
	}

	function sti_pg_get_data($connection_string, $data_source_name, $query)
	{
		$info = sti_pg_parse_connection_string($connection_string);
		$link = pg_connect("host=$info->host port=$info->port dbname=$info->database user=$info->user_id password=$info->password") or die("ServerError:Could not connect to host '$info->host', database '$info->database'.");
		
		$query = sti_pg_parse_parameters($query);
		$result = pg_query($link, $query) or die("ServerError:Data not found.");
		
		$xml_output = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n<Database>\n";
		
		$count = pg_num_fields($result);
		for ($fid = 0; $fid < $count; $fid++)
		{
			$columns[$fid] = pg_field_name($result, $fid);
		}
		
		while ($row = pg_fetch_assoc($result))
		{
			$xml_output .= "  <$data_source_name>\n";
			foreach($columns as $column)
			{
				$value = $row[$column];
				$value = str_replace("&", "&amp;", $value);
				$value = str_replace("<", "&lt;", $value);
				$value = str_replace(">", "&gt;", $value);

				$xml_output .= "    <$column>$value</$column>\n";
			}
			$xml_output .= "  </$data_source_name>\n";
		}
		
		$xml_output .= "</Database>\n";
		
		pg_free_result($result);
		pg_close($link);
		
		return $xml_output;
	}

?>