<?php

	function sti_get_xml_value($value, $key)
	{
		if (strpos($value, "<".$key.">") < 0 || strpos($value, "</".$key.">") < 0) return null;
		return substr(substr($value, 0, strpos($value, "</".$key.">")), strpos($value, "<".$key.">") + strlen($key) + 2);
	}
	
	function sti_strip($value)
	{
		if (get_magic_quotes_gpc() != 0)
		{
			if (is_array($value))
			{
				if (sti_array_is_associative($value))
				{
					foreach ($value as $k=>$v)
					{
						$tmp_val[$k] = stripslashes($v);
					}
					$value = $tmp_val;
				}
				else
				{				
					for ($j = 0; $j < sizeof($value); $j++)
					{
						$value[$j] = stripslashes($value[$j]);
					}
				}
			}
			else $value = stripslashes($value);
		}
		return $value;
	}
	
	function sti_array_is_associative($array)
	{
		if (is_array($array) && !empty($array))
		{
			for ($iterator = count($array) - 1; $iterator; $iterator--)
			{
				if (!array_key_exists($iterator, $array)) return true;
			}
			return !array_key_exists(0, $array);
		}
		return false;
	}
	
	function sti_create_connection_string($client_data)
	{
		$connection_string = sti_get_xml_value($client_data, "ConnectionString");
		$connection_string = sti_get_connection_string($connection_type, $connection_string);
		$user_name = sti_get_xml_value($client_data, "UserName");
		$password = sti_get_xml_value($client_data, "Password");
		if ($user_name != null && $password != null) $connection_string .= ";user=$user_name;password=$password";
		
		return $connection_string;
	}

	function sti_client_event_handler($client_key, $report_key, $client_data)
	{
		switch ($client_key)
		{
			// Loading ViewerFx
			case "ViewerFx":
				if (!isset($report_key)) $report_key = "null";
				$html_file = file_get_contents("viewer.html");
				return str_replace("#MARKER_REPORT_KEY#", $report_key, $html_file);
				break;

			// Loading DesignerFx
			case "DesignerFx":
				if (!isset($report_key)) $report_key = "null";
				$html_file = file_get_contents("designer.html");
				return str_replace("#MARKER_REPORT_KEY#", $report_key, $html_file);
				break;

			// Loading configuration
			case "LoadConfiguration":
				return sti_load_config("config.xml");
				break;

			// Loading a report when running the viewer or designer
 			case "LoadReport":
				return sti_get_report($client_data);
				break;

			// Loading a report by the specified URL
			// Used for debug. Does not work in the released version
			case "LoadReportFile":
				return file_get_contents($client_data);
				break;

			// Loading the requested localization file
			case "LoadLanguage":
				$localization_directory = sti_get_localization_directory();
				return file_get_contents($localization_directory."/".$client_data);
				break;

			// Saving a report
			case "SaveReport":
				return sti_save_report($client_data);
				break;
			
			// Test database connection
			case "TestConnection":
				$connection_type = sti_get_xml_value($client_data, "ConnectionType");
				$connection_string = sti_create_connection_string($client_data);
				
				switch ($connection_type)
				{
					case "StiSqlDatabase": return sti_mssql_test_connection($connection_string);
					case "StiMySqlDatabase": return sti_mysql_test_connection($connection_string);
					case "StiOdbcDatabase": return sti_odbc_test_connection($connection_string);
					case "StiPostgreSQLDatabase": return sti_pg_test_connection($connection_string);
					
					default: return "";
				}
				break;

			// Table columns query
			case "RetrieveColumns":
				$connection_type = sti_get_xml_value($client_data, "ConnectionType");
				$connection_string = sti_create_connection_string($client_data);
				
				switch ($connection_type)
				{
					case "StiSqlDatabase": return sti_mssql_get_columns($connection_string, $query);
					case "StiMySqlDatabase": return sti_mysql_get_columns($connection_string, $query);
					case "StiOdbcDatabase": return sti_odbc_get_columns($connection_string, $query);
					case "StiPostgreSQLDatabase": return sti_pg_get_columns($connection_string, $query);
					
					default: return "";
				}
				break;

			// Data query. Response to the client - data in the xml format
			case "LoadData":
				$connection_type = sti_get_xml_value($client_data, "ConnectionType");
				$connection_string = sti_create_connection_string($client_data);
				$data_source_name = sti_get_xml_value($client_data, "DataSourceName");
				$query = sti_get_xml_value($client_data, "Query");

				switch ($connection_type)
				{
					case "StiSqlDatabase": return sti_mssql_get_data($connection_string, $data_source_name, $query);
					case "StiMySqlDatabase": return sti_mysql_get_data($connection_string, $data_source_name, $query);
					case "StiOdbcDatabase": return sti_odbc_get_data($connection_string, $data_source_name, $query);
					case "StiPostgreSQLDatabase": return sti_pg_get_data($connection_string, $data_source_name, $query);
					
					default: return "";
				}
				break;
		}
	}
	
?>